﻿using System;
using Microsoft.SPOT;
using System.Net.Sockets;
using System.Net;
using System.Threading;
using System.Text;
using Microsoft.SPOT.Hardware;
using DeviceSolutions.SPOT.Hardware;

namespace TestTCPServer
{
    public static class Program
    {
        // Uscita digitale collegata al relé da attivare
        static OutputPort _relay = new OutputPort(Meridian.Pins.GPIO10, false);

        public static OutputPort Relay
        {
            get { return Program._relay; }
        }

        public static void Main()
        {
            // Creo il socket server, in ascolto sulla porta TCP 1234
            Socket serverSocket = new Socket(
                AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            IPEndPoint localEndPoint = new IPEndPoint(IPAddress.Any, 1234);

            serverSocket.Bind(localEndPoint);

            serverSocket.Listen(int.MaxValue);

            while (true)
            {
                // Resto in attesa del collegamento di un client...
                Socket clientSocket = serverSocket.Accept();

                // Creo un oggetto che processi la richiesta...
                new ClientSocketProcessor(clientSocket, true);
            }
        }
    }

    internal class ClientSocketProcessor
    {
        Socket _clientSocket;

        // Il costruttore memorizza il socket client e lancia un worker thread se richiesto
        public ClientSocketProcessor(Socket ClientSocket, bool async)
        {
            _clientSocket = ClientSocket;

            if (async)
            {
                new Thread(processRequest).Start();
            }
            else
            {
                processRequest();
            }
        }

        void processRequest()
        {
            string rxstring = string.Empty;

            // con il socket client...
            using (_clientSocket)
            {
                byte[] rx = new byte[1024];

                // fino a che è leggibile...
                while (_clientSocket.Poll(-1, SelectMode.SelectRead))
                {
                    if (_clientSocket.Available == 0) break;

                    int bytesread=_clientSocket.Receive(rx, _clientSocket.Available, SocketFlags.None);

                    // concateno le stringhe ricevute con quella corrente
                    rxstring += new string(Encoding.UTF8.GetChars(rx));

                    // se trovo il fine riga esco dal ciclo e processo il comando
                    if (rxstring.Length > 2 && 
                        rxstring[rxstring.Length - 2] == '\r' && 
                        rxstring[rxstring.Length - 1] == '\n') break;
                }

                // processing dei comandi riconosciuti
                switch (rxstring.ToLower())
                {
                    // comando "?"
                    case "?\n\r\n":
                    case "?\r\n":
                        // mando al client la stringa di "benvenuto"
                        _clientSocket.Send(Encoding.UTF8.GetBytes(".NET MF TCP Server 1.0\r\n"));
                        break;

                    case "on\n\r\n":
                    case "on\r\n":
                        // attivo il relay
                        Program.Relay.Write(true);

                        // mando al client la stringa di "feedback"
                        _clientSocket.Send(Encoding.UTF8.GetBytes("Relay ON\r\n"));
                        break;

                    case "off\n\r\n":
                    case "off\r\n":
                        // attivo il relay
                        Program.Relay.Write(false);

                        // mando al client la stringa di "feedback"
                        _clientSocket.Send(Encoding.UTF8.GetBytes("Relay OFF\r\n"));
                        break;

                    case "toggle\n\r\n":
                    case "toggle\r\n":
                        // attivo il relay
                        Program.Relay.Write(!Program.Relay.Read());

                        // mando al client la stringa di "feedback"
                        _clientSocket.Send(
                            Encoding.UTF8.GetBytes("Relay " + (Program.Relay.Read() ? "ON" : "OFF") + "\r\n"));
                        break;
                }
            }
        }
    }
}
